
local name, addon = ...;

Quests = {};

local questsInst;
--local onEvent = {};

function Quests:new(o)
  o = o or {};   -- create object if user does not provide one
  setmetatable(o, self);
  self.__index = self;
  
  self.pendingQuestData = {};
  self.areas = Areas:new();
  return o;
end

function getQuests()
  if (questsInst == nil) then
    questsInst = Quests:new();
  end
  return questsInst;
end

-- the idea was to find ! quests but here we get only ? 
function Quests:mapInfo()
	local playerMapID = C_Map.GetBestMapForUnit("player");
	local qsMap = C_QuestLog.GetQuestsOnMap(playerMapID);
	local i;	
	if (#qsMap > 0) then
		local onList = string.format("Quests %d on map", #qsMap);
		DEFAULT_CHAT_FRAME:AddMessage(onList);
		for i=1,#qsMap,1 do
			local inLog = false;
			local maxNumQuests = C_QuestLog.GetNumQuestWatches();
	        for j = 1,maxNumQuests,1 do
	           local questID = C_QuestLog.GetQuestIDForQuestWatchIndex(j);	-- https://wowpedia.fandom.com/wiki/API_C_QuestLog.GetQuestIDForQuestWatchIndex
	           if (qsMap[i].questID == questID) then
	              inLog = true;
	           end
            end
            if (not inLog) then
              local title = C_QuestLog.GetTitleForQuestID(qsMap[i].questID);	
		      onList = string.format("  %d %s", qsMap[i].questID, title);
		      DEFAULT_CHAT_FRAME:AddMessage(onList);
		    end
		end
	end
end

function Quests:createInfo(questID,complete)
	local info = {};
	local date = C_DateAndTime.GetCurrentCalendarTime();
	info["date"] = string.format("%d-%d-%d %d:%d:%d", date.year, date.month, date.monthDay, date.hour, date.minute, date.second);
	info["questID"] = questID;
	local playerMapID = C_Map.GetBestMapForUnit("player");
	info["mapID"] = playerMapID;
	local mapInfo = C_Map.GetMapInfo(playerMapID);
	info["mapName"] = mapInfo.name;
	info["complete"] = complete;
	local pos = C_Map.GetPlayerMapPosition(playerMapID, "player");
	if (pos ~= nil) then
		info["posX"] = pos.x*100.0;
		info["posY"] = pos.y*100.0;
	end
	local achievementID, storyMapID = C_QuestLog.GetZoneStoryInfo(playerMapID);
	if (achievementID ~= nil) then
		local numCrit = GetAchievementNumCriteria(achievementID);
		local i;
		for i = 1,numCrit,1 do
		  local criteriaString, criteriaType, completed, quantity, reqQuantity, charName, flags, assetID, quantityString, criteriaID, eligible  = GetAchievementCriteriaInfo(achievementID, i, false);
		  info["crit("..criteriaID..","..criteriaString..")"] = completed;
		end
		info["charName"] = charName;
    end
	return info;
end

function Quests:onQuestAccepted(frame, event, ...)
	local questID = ...; 
	local onList;
	local i;
	local isstory = self.areas:IsStoryline(questID); 
	if (isstory ~= nil) then
		onList = string.format("Quest %d is %s storyline", questID, isstory);
		onList = WrapTextInColorCode(onList, "ff00d000");
	else 
		onList = string.format("Quest %d is off storyline", questID);
		onList = WrapTextInColorCode(onList, "ffcccccc");
	end
	DEFAULT_CHAT_FRAME:AddMessage(onList);
	local info = self:createInfo(questID,false); 
	QuestInfo[#QuestInfo + 1] = info;
end

function Quests:onAddonLoaded(frame, event, ...)
	-- Our saved variables are ready at this point. If there are none, both variables will set to nil.
	if (QuestInfo == nil) then
		QuestInfo = {}; -- This is the first time this addon is loaded; initialize the info as table
	end
end

function Quests:onQuestTurnIn(frame, event, ...)
	local questID, xpReward, moneyReward = ...; 
	local info = self:createInfo(questID,true); 
	QuestInfo[#QuestInfo + 1] = info;
end

function Quests:onQuestDataLoaded(frame, event, ...)
  local questID, success = ...;
  if (self.pendingQuestData[questID] ~= nil) then
    self.pendingQuestData[questID] = nil;
    if (success) then
	  local title = C_QuestLog.GetTitleForQuestID(questID);	
	  msg = string.format(" %d %s", questID, title);
    else 
	  msg = string.format(" %d try again?", questID);
    end 
    msg = WrapTextInColorCode(msg, "ffc00000");	-- as these are missing show in red
    DEFAULT_CHAT_FRAME:AddMessage(msg);
  end  
end

function Quests:OnEvent(self, event, ...)
--  local args = {self, event, ...};
--  if (onEvent ~= nil) then
--    local func = onEvent[event];
--    if (func ~= nil) then
--	  func(unpack(args));
--	else  
--      local msg = string.format("Quests.onEvent %s is nil", event);
--      DEFAULT_CHAT_FRAME:AddMessage(msg);
--    end
--  else 
--    local msg = string.format("Quests.onEvent is nil");
--    DEFAULT_CHAT_FRAME:AddMessage(msg);
--  end
  local frame = self;
  local quests = getQuests();
  if (event == "ADDON_LOADED") then
    quests:onAddonLoaded(frame, event, ...);
  elseif (event == "QUEST_ACCEPTED") then
    quests:onQuestAccepted(frame, event, ...);
  elseif (event == "QUEST_TURNED_IN") then
    quests:onQuestTurnIn(frame, event, ...);
  elseif (event == "QUEST_DATA_LOAD_RESULT") then
    quests:onQuestDataLoaded(frame, event, ...);
  end
end

function Quests:OnLoad(frame)
  --onEvent = {};
  --onEvent["ADDON_LOADED"] = function(...)
  --  local args = {...};
  --  Quests:onAddonLoaded(unpack(args));
  --end
  frame:RegisterEvent("ADDON_LOADED");
  --onEvent["QUEST_ACCEPTED"] = function(...)
  --  local args = {...};
  --  Quests:onQuestAccepted(unpack(args));
  --end
  frame:RegisterEvent("QUEST_ACCEPTED");
  --onEvent["QUEST_TURNED_IN"] = function(...)
  --  local args = {...};
  --  Quests:onQuestTurnIn(unpack(args));
  --end
  frame:RegisterEvent("QUEST_TURNED_IN");
  --onEvent["QUEST_DATA_LOAD_RESULT"] = function(...)
  --  local args = {...};
  --  Quests:onQuestDataLaoded(unpack(args));
  --end
  frame:RegisterEvent("QUEST_DATA_LOAD_RESULT");
-- quest event
-- "QUEST_COMPLETE"
end

function Quests:ShowCriteria(achievementID)
	local i;
	local numCrit = GetAchievementNumCriteria(achievementID);
	for i = 1,numCrit,1 do
	  local criteriaString, criteriaType, completed, quantity, reqQuantity, charName, flags, assetID, quantityString, criteriaID, eligible  = GetAchievementCriteriaInfo(achievementID, i, false);
	  local msg = string.format("   %s quant %d/%d crit %d", criteriaString, quantity, reqQuantity, criteriaID);
	  if (completed) then
	    msg = WrapTextInColorCode(msg, "ff00d000");
      else
	    msg = WrapTextInColorCode(msg, "ffd00000");
	  end   
	  DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

function Quests:ShowAchivement4Map(playerMapID)
	local achievementID, storyMapID = C_QuestLog.GetZoneStoryInfo(playerMapID);
	local msg = string.format("Achivement %d map %d playMap %d", achievementID, storyMapID, playerMapID);
	DEFAULT_CHAT_FRAME:AddMessage(msg);
	if (achievementID ~= nil) then
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuild, wasEarnedByMe, earnedBy, isStatistic = GetAchievementInfo(achievementID);
		local cmpl;
		if (completed) then
		  cmpl = "Y";
		else
		  cmpl = "N";
		end   
		msg = string.format("Achivement %s completed %s", name, cmpl);
	else 
		msg = string.format("No Achivements here ...");
	end
	DEFAULT_CHAT_FRAME:AddMessage(msg);
	return achievementID;
end

function Quests:BuildQuestInfo(questID)
	local questInfos = C_QuestLog.GetQuestTagInfo(questID);	-- https://wowpedia.fandom.com/wiki/API_C_QuestLog.GetQuestTagInfo
	local info = "";
	local j;
	if (questInfos ~= nil) then
	  for j = 1,#questInfos,1 do
		 if (info ~= "") then
			info = info..", ";
		 end 
	     local questInfo = questInfos[j];
		 info = info..questInfo.tagID.." "..questInfo.tagName;
	  end 
	else 
	   info = "nil";
	end
	return info;
end

function Quests:ShowQuests(playerMapID)
	-- check watched/tracked quests	
	local maxNumQuests = C_QuestLog.GetNumQuestWatches();
	for i = 1,maxNumQuests,1 do
	  local questID = C_QuestLog.GetQuestIDForQuestWatchIndex(i);	-- https://wowpedia.fandom.com/wiki/API_C_QuestLog.GetQuestIDForQuestWatchIndex
	  local story = self.areas:IsStoryline(questID);
	  local onList = (story ~= nil);
	  local title = C_QuestLog.GetTitleForQuestID(questID);	
	  local questInfo = self:BuildQuestInfo(questID);
	  local completed = C_QuestLog.IsQuestFlaggedCompleted(questID);
	  local questType = C_QuestLog.GetQuestType(questID);	-- https://wowpedia.fandom.com/wiki/API_C_QuestLog.GetQuestType
	  local msg;
	  if (onList) then
	    msg = string.format("   %s id %d %s type 0x%x info %s", title, story, questID, questType, questInfo);
	    if (completed) then
			msg = WrapTextInColorCode(msg, "ff00d000");
	    else
			msg = WrapTextInColorCode(msg, "ff006000");
		end
	  else
	    msg = string.format("   %s id %d type 0x%x info %s", title, questID, questType, questInfo);
		msg = WrapTextInColorCode(msg, "ffcccccc");
	  end
	  DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

function Quests:ShowOpen(playerMapID)
  local map = self.areas.maps[playerMapID];
  if (map ~= nil) then
	local qs = map:quests();
	if (qs ~= nil) then
	  local i;
	  local cmpl = 0;
	  for i = 1,#qs,1 do
		if (C_QuestLog.IsQuestFlaggedCompleted(qs[i])) then
		  cmpl = cmpl + 1;
		end  
	  end	
	  local percent = ((cmpl / #qs) * 100.0);
	  local msg = string.format("Quests %d/%d %.1f%%", cmpl, #qs, percent);
	  DEFAULT_CHAT_FRAME:AddMessage(msg);
	  for i = 1,#qs,1 do
		local questID = qs[i];
		if (not C_QuestLog.IsQuestFlaggedCompleted(questID)) then
		  self.pendingQuestData[questID] = true;
		  C_QuestLog.RequestLoadQuestByID(questID);
		  -- fires QUEST_DATA_LOAD_RESULT
		end
	  end
	end
  end
end

-- "world" quests
function  Quests:mapTasks(playerMapID)
	local i;
	local taskPOIs = C_TaskQuest.GetQuestsForPlayerByMapID(playerMapID);
	local msg = string.format("Task %d", #taskPOIs);
	DEFAULT_CHAT_FRAME:AddMessage(msg);		
	for i = 1,#taskPOIs,1 do
		local taskPOI = taskPOIs[i];
		msg = string.format("   id %d %.1f/%.1f", taskPOI.questID, taskPOI.x*100, taskPOI.y*100);
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

-- e.g. these are spirit towers in terroka forest
function  Quests:pois(playerMapID)
	local i;
	local areaPoiIDs = C_AreaPoiInfo.GetAreaPOIForMap(playerMapID);
	local msg = string.format("POI");
	DEFAULT_CHAT_FRAME:AddMessage(msg);		
	for i = 1,#areaPoiIDs,1 do
		local poiInfo = C_AreaPoiInfo.GetAreaPOIInfo(playerMapID, areaPoiIDs[i]);
		msg = string.format("   %s id %d %.1f/%.1f", poiInfo.name, areaPoiIDs[i], poiInfo.position.x*100, poiInfo.position.y*100);
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

-- none found ...
function Quests:links(playerMapID) 
	local mapLinks = C_Map.GetMapLinksForMap(playerMapID);
	local i;
	local msg = string.format("Links %d", #mapLinks);
	DEFAULT_CHAT_FRAME:AddMessage(msg);		
	for i = 1,#mapLinks,1 do
		local mapLink = mapLinks[i];
		msg = string.format("   %s id %d %.1f/%.1f", mapLink.name, mapLink.areaPoiID, mapLink.position.x*100, mapLink.position.y*100);
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

function Quests:Slash(args, editbox)
    local i;
	local playerMapID = C_Map.GetBestMapForUnit("player");
	--local map = C_Map.GetPlayerMapPosition(playerMapID, "player")
    local achievementID = self:ShowAchivement4Map(playerMapID);
    local quests = getQuests();
    if (achievementID ~= nil) then
		quests:ShowCriteria(achievementID);
	end 
	--self:pois(playerMapID);
	--self:links(playerMapID);
	--self:mapTasks(playerMapID);
	if (string.upper(args) == "OPEN") then
		quests:ShowOpen(playerMapID);
	else
	   quests:ShowQuests(playerMapID);
	   quests:mapInfo();
	end
end
		
SLASH_QUESTS1 = "/quests"
SlashCmdList["QUESTS"] = function(args, editbox)
	Quests:Slash(args, editbox)
end
